@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <loudmouth/loudmouth.h>
@@ -438,7 +436,7 @@ void hk_message_in(const char *bjid, const char *resname,
       (buddy_getstatus(roster_usr->data, NULL) == offline &&
        buddylist_isset_filter()))
   {
-    update_roster = TRUE;
+    scr_update_roster();
   }
 
   g_free(bmsg);
@@ -569,8 +567,8 @@ void hk_statuschange(const char *bjid, const char *resname, gchar prio,
 
   roster_setstatus(bjid, rn, prio, status, status_msg, timestamp,
                    role_none, affil_none, NULL);
-  buddylist_build();
-  scr_draw_roster();
+  buddylist_defer_build();
+  scr_update_roster();
   hlog_write_status(bjid, timestamp, status, status_msg);
 
 #ifdef MODULES_ENABLE
