@@ -17,9 +17,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 #include <stdlib.h>
 #include <string.h>
@@ -44,7 +42,9 @@
 
 #define RECONNECTION_TIMEOUT    60L
 
-#define FINGERPRINT_LENGTH      16  //  Currently Loudmouth only supports MD5
+#ifndef LOUDMOUTH_USES_SHA256
+#define FINGERPRINT_LENGTH      16  // old loudmouth still uses MD5 :(
+#endif
 
 LmConnection* lconnection = NULL;
 static guint AutoConnection;
@@ -142,9 +142,8 @@ void xmpp_addbuddy(const char *bjid, const char *name, const char *group)
 
   roster_add_user(cleanjid, name, group, ROSTER_TYPE_USER, sub_pending, -1);
   g_free(cleanjid);
-  buddylist_build();
-
-  update_roster = TRUE;
+  buddylist_defer_build();
+  scr_update_roster();
 }
 
 void xmpp_updatebuddy(const char *bjid, const char *name, const char *group)
@@ -229,9 +228,9 @@ void xmpp_delbuddy(const char *bjid)
 
   roster_del_user(cleanjid);
   g_free(cleanjid);
-  buddylist_build();
+  buddylist_defer_build();
 
-  update_roster = TRUE;
+  scr_update_roster();
 }
 
 void xmpp_request(const char *fjid, enum iqreq_type reqtype)
@@ -711,6 +710,9 @@ static LmSSLResponse ssl_cb(LmSSL *ssl, LmSSLStatus status, gpointer ud)
     break;
   case LM_SSL_STATUS_UNTRUSTED_CERT:
     scr_LogPrint(LPRINT_LOGNORM, "Certificate is not trusted!");
+    // The user specified a fingerprint, let's wait for lm to check that
+    if (settings_opt_get("ssl_fingerprint"))
+      return LM_SSL_RESPONSE_CONTINUE;
     break;
   case LM_SSL_STATUS_CERT_EXPIRED:
     scr_LogPrint(LPRINT_LOGNORM, "Certificate has expired!");
@@ -723,13 +725,19 @@ static LmSSLResponse ssl_cb(LmSSL *ssl, LmSSLStatus status, gpointer ud)
                  "Certificate hostname does not match expected hostname!");
     break;
   case LM_SSL_STATUS_CERT_FINGERPRINT_MISMATCH: {
+#ifndef LOUDMOUTH_USES_SHA256
       char fpr[3*FINGERPRINT_LENGTH] = {0};
       fingerprint_to_hex(lm_ssl_get_fingerprint(ssl), fpr, FINGERPRINT_LENGTH);
+#endif
       scr_LogPrint(LPRINT_LOGNORM,
                 "Certificate fingerprint does not match expected fingerprint!");
+#ifndef LOUDMOUTH_USES_SHA256
       scr_LogPrint(LPRINT_LOGNORM, "Remote fingerprint: %s", fpr);
+#else
+      scr_LogPrint(LPRINT_LOGNORM, "Remote fingerprint: %s", lm_ssl_get_fingerprint(ssl));
+#endif
 
-      scr_LogPrint(LPRINT_LOGNORM, "Expected fingerprint: %s",
+      scr_LogPrint(LPRINT_LOGNORM, "Expect fingerprint: %s",
                    settings_opt_get("ssl_fingerprint"));
 
       return LM_SSL_RESPONSE_STOP;
@@ -752,10 +760,15 @@ static void connection_auth_cb(LmConnection *connection, gboolean success,
 {
   LmSSL *lssl;
   if ((lssl = lm_connection_get_ssl(connection)) != NULL) {
+#ifndef LOUDMOUTH_USES_SHA256
     char fpr[3*FINGERPRINT_LENGTH] = {0};
     fingerprint_to_hex(lm_ssl_get_fingerprint(lssl), fpr, FINGERPRINT_LENGTH);
     scr_LogPrint(LPRINT_LOGNORM, "Connection established.\n"
                  "Remote fingerprint: %s", fpr);
+#else
+    scr_LogPrint(LPRINT_LOGNORM, "Connection established.\n"
+                 "Remote fingerprint: %s", lm_ssl_get_fingerprint(lssl));
+#endif
   }
 
   if (success) {
@@ -863,7 +876,7 @@ static void connection_close_cb(LmConnection *connection,
   // Reset carbons
   carbons_reset();
   // Update display
-  update_roster = TRUE;
+  scr_update_roster();
   scr_update_buddy_window();
 
   if (!reason)
@@ -913,7 +926,7 @@ static void handle_state_events(const char *bjid,
   }
 
   buddy_resource_setevents(sl_buddy->data, resource, xep85->last_state_rcvd);
-  update_roster = TRUE;
+  scr_update_roster();
 #endif
 }
 
@@ -972,8 +985,8 @@ static void gotmessage(LmMessageSubType type, const char *from,
       buddy_settype(room_elt->data, ROSTER_TYPE_ROOM);
     }
 
-    buddylist_build();
-    scr_draw_roster();
+    buddylist_defer_build();
+    scr_update_roster();
     goto gotmessage_return;
   }
 
@@ -1123,6 +1136,13 @@ static LmHandlerResult handle_messages(LmMessageHandler *handler,
     LmMessageNode *xenc;
     const char *carbon_name = x->name;
     carbons = TRUE;
+
+    // Check envelope JID for carbon messages
+    if (!jid_equal(lm_connection_get_jid(lconnection), bjid)) {
+      scr_LogPrint(LPRINT_LOGNORM, "Received invalid carbon copy from %s.", bjid);
+      goto handle_messages_return;
+    }
+
     // Go 1 level deeper to the forwarded message
     x = lm_message_node_find_xmlns(x, NS_FORWARD);
     if (x)
@@ -1680,7 +1700,7 @@ static LmHandlerResult handle_s10n(LmMessageHandler *handler,
     /* The subscription request has been denied or a previously-granted
        subscription has been cancelled */
     roster_unsubscribed(from);
-    update_roster = TRUE;
+    scr_update_roster();
     buf = g_strdup_printf("<%s> has cancelled your subscription to "
                           "their presence updates", from);
     scr_WriteIncomingMessage(r, buf, 0, HBB_PREFIX_INFO, 0);
@@ -1692,7 +1712,7 @@ static LmHandlerResult handle_s10n(LmMessageHandler *handler,
   }
 
   if (newbuddy)
-    update_roster = TRUE;
+    scr_update_roster();
   g_free(r);
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
@@ -1757,7 +1777,9 @@ gint xmpp_connect(void)
 {
   const char *userjid, *password, *resource, *servername, *ssl_fpr;
   char *dynresource = NULL;
+#ifndef LOUDMOUTH_USES_SHA256
   char fpr[FINGERPRINT_LENGTH] = {0};
+#endif
   const char *proxy_host;
   const char *resource_prefix = PACKAGE_NAME;
   char *fjid;
@@ -1902,6 +1924,7 @@ gint xmpp_connect(void)
     port = (ssl ? LM_CONNECTION_DEFAULT_PORT_SSL : LM_CONNECTION_DEFAULT_PORT);
   lm_connection_set_port(lconnection, port);
 
+#ifndef LOUDMOUTH_USES_SHA256
   if (ssl_fpr && (!hex_to_fingerprint(ssl_fpr, fpr, FINGERPRINT_LENGTH))) {
     scr_LogPrint(LPRINT_LOGNORM, "** Please set the fingerprint in the format "
                  "97:5C:00:3F:1D:77:45:25:E2:C5:70:EC:83:C8:87:EE");
@@ -1909,6 +1932,9 @@ gint xmpp_connect(void)
   }
 
   lssl = lm_ssl_new((ssl_fpr ? fpr : NULL), ssl_cb, NULL, NULL);
+#else
+  lssl = lm_ssl_new(ssl_fpr, ssl_cb, NULL, NULL);
+#endif
   if (lssl) {
 #ifdef HAVE_LM_SSL_CIPHER_LIST
     const char *ssl_ciphers = settings_opt_get("ssl_ciphers");
@@ -1916,7 +1942,10 @@ gint xmpp_connect(void)
 #endif
 #ifdef HAVE_LM_SSL_CA
     const char *ssl_ca = settings_opt_get("ssl_ca");
-    lm_ssl_set_ca(lssl, ssl_ca);
+    char *ssl_ca_xp;
+    ssl_ca_xp = expand_filename(ssl_ca);
+    lm_ssl_set_ca(lssl, ssl_ca_xp);
+    g_free(ssl_ca_xp);
 #endif
     lm_ssl_use_starttls(lssl, !ssl, tls);
     lm_connection_set_ssl(lconnection, lssl);
@@ -2044,7 +2073,7 @@ void xmpp_setstatus(enum imstatus st, const char *recipient, const char *msg,
     // We'll have to update the roster if we switch to/from offline because
     // we don't know the presences of buddies when offline...
     if (mystatus == offline || st == offline)
-      update_roster = TRUE;
+      scr_update_roster();
 
     if (isonline || mystatus || st)
 #ifdef WITH_DEPRECATED_STATUS_INVISIBLE
@@ -2096,7 +2125,7 @@ void xmpp_setprevstatus(void)
 
 //  send_storage(store)
 // Send the node "store" to update the server.
-// Note: the sender should check we're online.
+// Note: the caller should check we're online.
 void send_storage(LmMessageNode *store)
 {
   LmMessage *iq;
@@ -2310,17 +2339,18 @@ void xmpp_set_storage_bookmark(const char *roomid, const char *name,
     if (group && *group)
       lm_message_node_add_child(x, "group", group);
     changed = TRUE;
-    scr_LogPrint(LPRINT_LOGNORM, "Updating bookmarks...");
   }
 
   if (!changed)
     return;
 
-  if (xmpp_is_online())
+  if (xmpp_is_online()) {
     send_storage(bookmarks);
-  else
+    scr_LogPrint(LPRINT_LOGNORM, "Bookmarks updated.");
+  } else {
     scr_LogPrint(LPRINT_LOGNORM,
                  "Warning: you're not connected to the server.");
+  }
 }
 
 static struct annotation *parse_storage_rosternote(LmMessageNode *notenode)
