@@ -18,9 +18,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <string.h>
@@ -584,6 +582,20 @@ LmHandlerResult handle_iq_roster(LmMessageHandler *h, LmConnection *c,
   int need_refresh = FALSE;
   guint roster_type;
 
+  const gchar *from = lm_message_get_from(m);
+
+  if (from) {
+    const gchar *self_jid = lm_connection_get_jid(c);
+    gchar *servername = get_servername(self_jid, "");
+    if ((!jid_equal(self_jid, from)) &&
+       (!servername || strcasecmp(from, servername))) {
+      scr_LogPrint(LPRINT_LOGNORM, "Received invalid roster IQ request");
+      g_free(servername);
+      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+    }
+    g_free(servername);
+  }
+
   y = lm_message_node_find_child(lm_message_node_find_xmlns(m->node, NS_ROSTER),
                                  "item");
   for ( ; y; y = y->next) {
@@ -656,8 +668,8 @@ LmHandlerResult handle_iq_roster(LmMessageHandler *h, LmConnection *c,
     lm_message_unref(result);
   }
 
-  buddylist_build();
-  update_roster = TRUE;
+  buddylist_defer_build();
+  scr_update_roster();
   if (need_refresh)
     scr_update_buddy_window();
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
