@@ -15,9 +15,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <string.h>
@@ -672,8 +670,8 @@ static void storage_bookmarks_parse_conference(LmMessageNode *node)
   }
   g_free(bjid);
 
-  buddylist_build();
-  update_roster = TRUE;
+  buddylist_defer_build();
+  scr_update_roster();
 }
 
 static LmHandlerResult cb_storage_bookmarks(LmMessageHandler *h,
